! CH4.H2O intermolecular PES
! form Michael Metz and Krzysztof Szalewicz (Edit Matyus's e-mail)

! coefficient (fit_inter.dat) versions:
! first version : 2016.11.19. (test version, smaller basis)
! second version: 2016.12.18. (first version)
! third version : 2018.06.28. (final version, published)

! forged and implemented in GENIUSH: 2017.01.04. by Janos Sarka

! Author comments

!
!**************************** REQUIRED INPUT *******************************
!
! fit.in : Contains all information about PES in the following format:
! > [order of polynomial], [number of VDW terms per site pair], [lowest VDW term]
! > [number of sites in monomer A], [number of sites in monomer B]
! > [site type 1 of monomer A], [site type 2 of monomer A], ...
! > [site type 1 of monomer B], [site type 2 of monomer B], ...
! > 0
! > [charge of site type 1], [polarizability of site type 1]
! > [charge of site type 2], [polarizability of site type 2]
! > ...
! > [number of site pair types]
! > [site type a], [site type b], [beta], [alpha], [a_1], [a_2], ..., [C_6], [C_8], ..., [A_12], [delta_1], [delta_6], [delta_8], ..., [delta_pol]
! > ...
! All units are in Angstrom and kcal/mol, except charges which are in atomic units
!
! standard input : The geometry is read in from standard input in the following format:
! > [x position of monomer A site 1], [y position of monomer A site 1], [z position of monomer A site 1]
! > [x position of monomer A site 2], [y position of monomer A site 2], [z position of monomer A site 2]
! > ...
! > [x position of monomer B site 1], [y position of monomer B site 1], [z position of monomer B site 1]
! > [x position of monomer B site 2], [y position of monomer B site 2], [z position of monomer B site 2]
! > ...
! Coordinates are given in bohrs. The monomers may then be shifted and rotated using moveGeo function.
!
!***************************************************************************


!***************************************************************************

! routine modified by Janos Sarka (to work in parallel)
! last change: 2017.09.19.
! (more digits for CH4 sites included, 8 --> 15)

! Notes:
! 1) this routine handles all 3 dimers (MW, MM, WW), see whichmolecule variable
! 2) the potential can be called parallelly using openmp, 
!    but it needs to be called once before that in serially to initialize the PES (to read the parameters).
! 3) note that the size of the arrays sitCH4 and sitH2O is hard-coded in the site generation part of the code
! 4) The coefficient files (fit_inter.dat.CH4-H2O, fit_inter.dat.CH4-CH4, fit_inter.dat.H2O-H2O for MW, MM and WW)
!    need to be renamed to 'fit_inter.dat' in order to use them with this routine.

! Calling the PES:
! call ch4h2o_pes(natom,rri,V)

! input/output parameters:
! 1) natom: number of atoms
! 2) rr(natom,3): geometry in bohr
! 3) Vpot: the potential in Eh


! The PESs where used with frozen monomers parameters:
! rOH = 0.97565d0 ! in Angstrom
! aHOH = -0.24919700d0 ! cos(104.430)
! rCH = 1.110020996d0 ! in Angstrom

! Shifts needed to be applied to get 0.0 Eh in the global minima (using frozen monomer parameters):
! Vpot = Vpot + Vshift

! MW
! Vshift = 1.675898680179050d-3 ! -367.817242749785 cm-1

! MM
! Vshift = 8.667370050737186d-4 ! -190.226783495863 cm-1

! WW
! Vshift = 8.159748906129272d-3 ! -1790.85787206563 cm-1

!***************************************************************************


subroutine ch4h2o_pes(natom,rr,Vpot)
!
implicit none

real(8), parameter :: bohr2A = 0.529177249d0, h2kcal = 627.510d0
real(8), parameter :: deg2rad = 0.01745329252d0
real(8), parameter :: ehcm = 219474.63d0

integer :: i,j,k,l
!real(8), save, allocatable :: geo(:,:,:), geo0(:,:,:), geo1(:,:,:), geoMon(:,:,:)
!real(8), save, allocatable :: sitCH4(:,:), sitH2O(:,:)
real(8), save, allocatable :: charge(:), pz(:)
real(8), save, allocatable :: alpha(:), beta(:), delta1(:), deltaP(:), deltan(:,:), chargeProd(:), a(:,:), C(:,:), A12(:)
!!real(8), save, allocatable :: rAB(:,:)
integer, save, allocatable :: stype(:,:)
integer, save, allocatable :: typeMap(:,:) ! Map from pairs of site types to site pair types
integer, save :: oPoly, nDisp, cStart
integer, save :: nstype, nsptype, nsite(2), maxnsite
logical, save :: incPol

real(8) :: val, minVal
!integer :: natom1, natom2
 
!save oPoly, nDisp, cStart, nsite, maxnsite, nstype, nsptype
!save stype, charge, pz, typeMap, chargeProd, beta, alpha, a, C, A12, delta1, deltan, deltaP
!save geo, geo0, rAB

real(8) :: sum, ener1, ener2, mean
integer :: nneg, ndat, gInd1, gInd2, ival1, ival2
real(8) :: rbuf(100)
character(100) :: str, testfile
real(8) :: com(3), monCom(3)
integer :: centMon
integer :: cellMon(4)

! input variables
integer :: natom
real(8) :: rr(natom,3)
real(8) :: Vpot

integer :: whichmolecule
logical :: pot_begin
data pot_begin/.true./
save pot_begin

! choose which dimer's PES to use
whichmolecule=3 ! 1: MW, 2: MM, 3: WW

!write(*,*) "rr"
!write(*,*) rr

! Read and initialize fit from file
if (pot_begin) then
  call readFit('fit_inter.dat')
  write(1,*) "PES parameters have been read from fit_inter.dat!"
  pot_begin=.false.
end if
!geo=0.d0
!rAB=0.d0
!write(*,*) "oPoly, nDisp, cStart, nsite, maxnsite, nstype, nsptype"
!write(*,*) oPoly, nDisp, cStart, nsite, maxnsite, nstype, nsptype
!write(*,*) "stype, charge, pz, typeMap, chargeProd, beta, alpha, a, C, A12, delta1, deltan, deltaP"
!write(*,*) stype, charge, pz, typeMap, chargeProd, beta, alpha, a, C, A12, delta1, deltan, deltaP
!write(*,*) "geo"
!write(*,*) geo
!write(*,*) "rAB"
!write(*,*) rAB


! energy value
!write(*,*) evalEnergy()*h2kcal," kcal/mol"
!write(*,*) evalEnergy()*ehcm," cm-1"

!write(*,*) "a",Vpot 
Vpot = evalEnergy(natom,rr)
!write(*,*) "b",Vpot

contains


! Calculate value of PES
function evalEnergy(natom,rr)
use molecules

  real(8) :: evalEnergy
  
  integer :: ii,jj,kk
  integer :: sptype
  real(8) :: Eexp, Eels, Evdw, Epol
  real(8) :: sum, rPow, rInv, r

  integer :: natom
  real(8) :: rr(natom,3)

  integer :: natom1, natom2
  real(8) :: geo(2,maxnsite,3)
  real(8) :: sitCH4(nsite(1),3), sitH2O(nsite(2),3)
  real(8) :: rAB(nsite(1),nsite(2))
 
select case(whichmolecule)
  ! CH4.H2O
  case(1)
     natom1 = 5
     natom2 = 3
  ! CH4.CH4
  case(2)
     natom1 = 5
     natom2 = 5
  ! H2O.H2O
  case(3)
     natom1 = 3
     natom2 = 3
end select

! Geometry from input rr
do j = 1, natom1
   sitCH4(j,1:3) = rr(j,1:3)
enddo
do j = 1, natom2
   sitH2O(j,1:3) = rr(j+natom1,1:3)
enddo

! determine the sites
! The sites_ch4 and sites_h2o requires bohr!!!

!call sites_ch4(sitCH4)
!do i = 1,9
!   write(*,'(3F20.10)') sitCH4(i,1:3)
!enddo
!write(*,*)

!call sites_h2o(sitH2O)
!do i = 1,26
!   write(*,'(3F20.10)') sitH2O(i,1:3)
!enddo
!write(*,*)

! Universal solution (2017.03.09.)
select case(whichmolecule)
   ! CH4.H2O
   case(1)
      call sites_ch4(sitCH4)
      call sites_h2o(sitH2O)
   ! CH4.CH4
   case(2)
      call sites_ch4(sitCH4)
      call sites_ch4(sitH2O)
   ! H2O.H2O
   case(3)
      call sites_h2o(sitCH4)
      call sites_h2o(sitH2O)
end select

! define geo array
do j = 1, nsite(1)
   geo(1,j,1:3) = sitCH4(j,1:3)
enddo


do j = 1, nsite(2)
   geo(2,j,1:3) = sitH2O(j,1:3)
enddo


  do ii = 1,nsite(1)
     do jj = 1,nsite(2)
        rAB(ii,jj) = sqrt( &
             (geo(1,ii,1) - geo(2,jj,1))**2 + &
             (geo(1,ii,2) - geo(2,jj,2))**2 + &
             (geo(1,ii,3) - geo(2,jj,3))**2)
     enddo
  enddo
  
  Eexp = 0.0d0
  Eels = 0.0d0
  Evdw = 0.0d0
  Epol = 0.0d0
  
  do ii = 1,nsite(1)
     do jj = 1,nsite(2)
        sptype = typeMap(stype(1,ii),stype(2,jj))
        r = rAB(ii,jj)
        
        if (beta(sptype) > 0.0d0) then
           sum = 1.0d0
           rPow = 1.0d0
           do kk = 1,oPoly
              rPow = rPow*r
              sum = sum + rPow*a(sptype,kk)
           enddo
           Eexp = Eexp + sum*exp(alpha(sptype) - beta(sptype)*r)
        endif
        Eexp = Eexp + A12(sptype)/r**12
        
        rInv = 1.0d0/r
        Eels = Eels + chargeProd(sptype)*rInv*TT(1, delta1(sptype), r)
        
        rInv = rInv*rInv
        rPow = rInv**((cStart - 2)/2)
        do kk = 1,nDisp
           rPow = rPow*rInv
           Evdw = Evdw - C(sptype,kk)*rPow*TT(cStart + 2*(kk-1), deltan(sptype,kk), r)
        enddo
        
     enddo
  enddo
  
  if (incPol) Epol = polEnergy(geo,rAB)
  
  evalEnergy = Eexp + Eels + Evdw + Epol
  !write(*,*) Eexp, Eels, Evdw, Epol
  !write(*,*) Evdw*h2kcal

return
end function


! Calculate polarization energy for given configuration
function polEnergy(geo,rAB)
  real(8) :: polEnergy
  
  integer :: ii,jj,kk,ll
  real(8) :: T(nsite(1),nsite(2),3,3)
  real(8) :: Efield(2,maxnsite,3), fieldij(3)
  real(8) :: mu(2,maxnsite,3), muNext(2,maxnsite,3)
  real(8) :: rVect(3), r, dampFact
  real(8) :: val, iterChange, diff(3), pz1, pz2
  integer :: sptype, imon1, imon2, sitea, siteb
  
  real(8) :: geo(2,maxnsite,3)
  real(8) :: rAB(nsite(1),nsite(2))

  ! Calculate E field due to point charges
  Efield(:,:,:) = 0.0d0
  do ii = 1,nsite(1)
     do jj = 1,nsite(2)
        rVect = geo(1,ii,:) - geo(2,jj,:)
        r = rAB(ii,jj)
        sptype = typeMap(stype(1,ii),stype(2,jj))
        dampFact = TT(1, delta1(sptype), r)
        
        fieldij = rVect(:)*dampFact/r**3
        Efield(1,ii,:) = Efield(1,ii,:) + fieldij*charge(stype(2,jj))
        Efield(2,jj,:) = Efield(2,jj,:) + -fieldij*charge(stype(1,ii))
     enddo
  enddo
  
  ! Calculate dipole tensor
  do ii = 1,nsite(1)
     do jj = 1,nsite(2)
        if (pz(stype(1,ii)) > 1.0d-50 .and. pz(stype(2,jj)) > 1.0d-50) then
           rVect = geo(1,ii,:) - geo(2,jj,:)
           r = rAB(ii,jj)
           sptype = typeMap(stype(1,ii),stype(2,jj))
           
           dampFact = TT(3, deltaP(sptype), r)
           do kk = 1,3
              do ll = 1,3
                 
                 val = 3.0d0*dampFact*rVect(kk)*rVect(ll)/r**5
                 if (kk == ll) then
                    val = val - dampFact/r**3
                 endif
                 
                 T(ii,jj,kk,ll) = val
              enddo
           enddo
           
        else
           T(ii,jj,:,:) = 0.0d0
        endif
        
     end do
  end do
  
  ! Solve for induced dipoles iteratively
  mu = 0.0d0
  do ii = 1,20
     iterChange = 0.0d0
     
     do imon1 = 1,2
        
        if (imon1 == 1) then
           imon2 = 2
        else
           imon2 = 1
        endif
        
        do jj = 1,nsite(imon1)
           if (pz(stype(imon1,jj)) > 1.0d-50) then
              
              muNext(imon1,jj,:) = Efield(imon1,jj,:)
              
              do kk = 1,nsite(imon2)
                 if (pz(stype(imon2,kk)) > 1.0d-50) then
                    
                    if (imon1 == 1) then
                       sitea = jj
                       siteb = kk
                    else
                       sitea = kk
                       siteb = jj
                    endif
                    
                    muNext(imon1,jj,:) = muNext(imon1,jj,:) + matmul(T(sitea,siteb,:,:), mu(imon2,kk,:))
                 endif
              enddo
              
              muNext(imon1,jj,:) = muNext(imon1,jj,:)*pz(stype(imon1,jj))
              
              diff = muNext(imon1,jj,:) - mu(imon1,jj,:)
              iterChange = iterChange + dot_product(diff, diff)
              
              mu(imon1,jj,:) = muNext(imon1,jj,:)
              
           endif
        enddo
     enddo
     
     if (iterChange < maxnsite*1.0d-16) exit
  end do
  
  ! Find final polarization energy
  polEnergy = 0.0
  do ii = 1,2
     do jj = 1,nsite(ii)
        polEnergy = polEnergy + -0.5d0*dot_product(Efield(ii,jj,:), mu(ii,jj,:))
     end do
  end do
  
end function


! Tang-Toennies damping funciton
function TT(n, delta, r)
  integer :: n
  real(8) :: delta
  real(8) :: r
  real(8) :: TT
  
  integer :: ii
  real(8) :: rScaled, sum, fact, rtoii
  
  rScaled = r*delta
  rtoii = 1.0
  fact = 1.0
  sum = 1.0
  do ii = 1,n
     fact = fact*ii
     rtoii = rtoii*rScaled
     sum = sum + rtoii/fact
  end do
  
  TT = 1.0 - exp(-rScaled)*sum
end function


! Read in PES information from given file
subroutine readFit(filename)
  character(*) :: filename
  
  integer :: ii, jj
  integer :: index, ncol, noagroup
  integer :: iBuf(100)
  real(8) :: rBuf(100)
  character(1000) :: line
  
  !if (allocated(geo)) then
  !   deallocate(geo,geo0,stype,charge,pz,typeMap,alpha,beta,delta1,deltaP,deltan,C,A12,a,chargeProd,rAB)
  !   write(*,*) "PES variables initially deallocated!"
  !endif
  
  open(100,file=filename,status='old')
  
  read(100,*) oPoly, nDisp, cStart
  
  read(100,*) nsite(1:2)
  maxnsite = max(nsite(1), nsite(2))
  
  !allocate( geo(2,maxnsite,3) )
  !allocate( geo0(2,maxnsite,3) )
  allocate( stype(2,maxnsite) )
  stype = 0

  !allocate(sitCH4(nsite(1),3))
  !allocate(sitH2O(nsite(2),3))
 
  do ii = 1,2
     read(100,'(A)') line
     call splitLine(line, iBuf)
     stype(ii,1:nsite(ii)) = iBuf(1:nsite(ii))
  enddo
  
  read(100,*) noagroup
  if (noagroup /= 0) then
     write(*,*) 'ERROR: Number of optimized off-atomic site groups must equal 0'
     stop
  endif
  
  nstype = max(maxval(stype(1,:)), maxval(stype(2,:)))
  
  allocate( charge(nstype), pz(nstype) )
 ! 
  incPol = .false.
  do ii = 1,nstype
     read(100,*) rBuf(1),rBuf(2)
     charge(ii) = rBuf(1)
     pz(ii) = rBuf(2)
     if (pz(ii) > 1.0d-20) incPol = .true.
  enddo
  
  read(100,*) nsptype
  
  allocate( typeMap(nstype,nstype) )
  allocate( alpha(nsptype), beta(nsptype), delta1(nsptype), deltaP(nsptype), A12(nsptype) )
  allocate( deltan(nsptype,nDisp), C(nsptype,nDisp) )
  allocate( a(nsptype,oPoly) )
  allocate( chargeProd(nsptype) )
  
  typeMap(:,:) = 0
  
  do ii = 1,nsptype
     ncol = 2 + oPoly + nDisp + 1 + 1 + nDisp
     if (incPol) ncol = ncol + 1
     read(100,*) iBuf(1), iBuf(2), rBuf(1:ncol)
     index = 1
     
     typeMap(iBuf(1),iBuf(2)) = ii
     typeMap(iBuf(2),iBuf(1)) = ii
     
     chargeProd(ii) = charge(iBuf(1))*charge(iBuf(2))
     
     beta(ii) = rBuf(index)*bohr2A
     index = index + 1
     alpha(ii) = rBuf(index) - log(h2kcal)
     index = index + 1
     do jj = 1,oPoly
        a(ii,jj) = rBuf(index)*bohr2A**jj
        index = index + 1
     end do
     do jj = 1,nDisp
        C(ii,jj) = rBuf(index)/h2kcal/bohr2A**(cStart + 2*(jj-1))
        index = index + 1
     enddo
     A12(ii) = rBuf(index)/h2kcal/bohr2A**12
     index = index + 1
     delta1(ii) = rBuf(index)*bohr2A
     index = index + 1
     do jj = 1,nDisp
        deltan(ii,jj) = rBuf(index)*bohr2A
        index = index + 1
     enddo
     if (incPol) then
        deltaP(ii) = rBuf(index)*bohr2A
        index = index + 1
     endif
  enddo
  
  !allocate( rAB(nsite(1),nsite(2)) )
  
  close(100)
  
end subroutine


! Parses a string to an array of ints
subroutine splitLine(line, arr)
  character(*) :: line
  integer :: arr(:)
  
  integer :: ii
  integer :: start, arrIndex
  logical :: blank, blankChar
  
  arr(:) = 0.0d0
  
  blank = .true.
  arrIndex = 1
  do ii = 1,len(line)
     blankChar = index('0123456789+-', line(ii:ii)) == 0
     
     if (blank) then
        if (.not. blankChar) then
           start = ii
           blank = .false.
        endif
     else
        if (blankChar .or. ii == len(line)) then
           read(line(start:ii-1),*) arr(arrIndex)
           arrIndex = arrIndex + 1
           blank = .true.
        endif
     endif
  enddo  
end subroutine


! Rotate and shift a geometry
function moveGeo(geo, x, y, z, aa, ab, ag)
  real(8) :: geo(maxnsite,3)
  real(8) :: x, y, z, aa, ab, ag
  real(8) :: moveGeo(size(geo),3)
  
  integer :: ii
  real(8) :: rotMat(3,3), shiftVect(3)
  
  shiftVect = [x,y,z]
  
  rotMat(1,1) = cos(aa)*cos(ab)*cos(ag) - sin(aa)*sin(ag)
  rotMat(1,2) = -cos(aa)*cos(ab)*sin(ag) - sin(aa)*cos(ag)
  rotMat(1,3) = cos(aa)*sin(ab)
  rotMat(2,1) = sin(aa)*cos(ab)*cos(ag) + cos(aa)*sin(ag)
  rotMat(2,2) = -sin(aa)*cos(ab)*sin(ag) + cos(aa)*cos(ag)
  rotMat(2,3) = sin(aa)*sin(ab)
  rotMat(3,1) = -sin(ab)*cos(ag)
  rotMat(3,2) = sin(ab)*sin(ag)
  rotMat(3,3) = cos(ab)
  
  do ii = 1,size(geo)
     moveGeo(ii,:) = matmul(rotMat(:,:), geo(ii,:)) + shiftVect(:)
  enddo
  
end function


subroutine setConfig(R, betaA, gammaA, alphaB, betaB, gammaB, geo0, geo)
  real(8) :: R, betaA, gammaA, alphaB, betaB, gammaB
  real(8) :: geo0(2,maxnsite,3), geo(2,maxnsite,3)
  
  geo(1,:,:) = moveGeo(geo0(1,:,:), 0.0d0, 0.0d0, 0.0d0,    0.0d0,          betaA*deg2rad, gammaA*deg2rad)
  geo(2,:,:) = moveGeo(geo0(2,:,:), 0.0d0, 0.0d0, R/bohr2A, alphaB*deg2rad, betaB*deg2rad, gammaB*deg2rad)
end subroutine


! Counts the number of lines in a file, returns 0 if file does not exist
function nLines(filename)
  implicit none
  character(*) :: filename
  integer :: nLines
  
  integer, parameter :: nLinesUnit = 100
  
  nLines = 0
  open(nLinesUnit, file = filename, status = 'old', ERR=10)
  
  do
     read(nLinesUnit, *, END=10)
     nLines = nLines + 1
  end do
  
10 close(nLinesUnit)
end function nLines


end subroutine


subroutine sites_ch4 (sitCH4)
  implicit none
  
  real(8), parameter :: A2bohr = 1.88972598858d0
  integer :: i, j, k
  real(8) :: sitCH4(9,3)
  real(8) :: unitvect(3,3)
  real(8) :: OACoor(4,3)
  real(8) :: rCH, scaleFact
  
  ! z-axis is in direction from C1 to H1
  unitvect(3,1:3) = sitCH4(2,1:3) - sitCH4(1,1:3)
  unitvect(3,1:3) = unitvect(3,1:3)/sqrt(unitvect(3,1)**2 + unitvect(3,2)**2 + unitvect(3,3)**2)
  ! y-axis is in direction from C1 to H2, minus the projection onto z-axis
  unitvect(2,1:3) = sitCH4(3,1:3) - sitCH4(1,1:3)
  unitvect(2,1:3) = unitvect(2,1:3) - unitvect(3,1:3)*( &
       unitvect(2,1)*unitvect(3,1) + unitvect(2,2)*unitvect(3,2) + unitvect(2,3)*unitvect(3,3))
  unitvect(2,1:3) = unitvect(2,1:3)/sqrt(unitvect(2,1)**2 + unitvect(2,2)**2 + unitvect(2,3)**2)
  ! x-axis is y cross z
  unitvect(1,1) = unitvect(2,2)*unitvect(3,3) - unitvect(2,3)*unitvect(3,2)
  unitvect(1,2) = unitvect(2,3)*unitvect(3,1) - unitvect(2,1)*unitvect(3,3)
  unitvect(1,3) = unitvect(2,1)*unitvect(3,2) - unitvect(2,2)*unitvect(3,1)
  
  ! Off-atomic site locations in the original coordinate system
  OACoor(1,1:3) = [ 0.000000000000000d0, 0.000000000000000d0,-1.384561800000000d0]
  OACoor(2,1:3) = [ 0.000000000000000d0,-1.305377383669136d0, 0.461520600000000d0]
  OACoor(3,1:3) = [-1.130489975783138d0, 0.652688691834568d0, 0.461520600000000d0]
  OACoor(4,1:3) = [ 1.130489975783138d0, 0.652688691834568d0, 0.461520600000000d0]
  !OACoor(1,1:3) = [ 0.00000000d0,-0.00000000d0,-1.38456180d0]
  !OACoor(2,1:3) = [ 0.00000000d0,-1.30536594d0, 0.46155297d0]
  !OACoor(3,1:3) = [-1.13048006d0, 0.65268282d0, 0.46155297d0]
  !OACoor(4,1:3) = [ 1.13048006d0, 0.65268282d0, 0.46155297d0]
  
  ! Compute C-H distance
  rCH = 0.0d0
  do i = 2,5
     rCH = rCH + sqrt((sitCH4(1,1) - sitCH4(i,1))**2 + &
          (sitCH4(1,2) - sitCH4(i,2))**2 + (sitCH4(1,3) - sitCH4(i,3))**2)
  enddo
  rCH = rCH/4
  
  ! Scale factor is the ratio of C-H distance to that used in the original geometry
  !scaleFact = rCH/2.09237d0 !2.07680886d0
  scaleFact = rCH/2.092384930000000d0
  
  ! Compute off-atomic site locations in new coordinate system, with origin at C1
  do i = 1,4
     sitCH4(i+5,1:3) = sitCH4(1,1:3)
     do j = 1,3
        sitCH4(i+5,1:3) = sitCH4(i+5,1:3) + scaleFact*unitvect(j,1:3)*OACoor(i,j)
     enddo
  enddo
  
  !do i = 1,9
  !   write(*,'(3F20.10)') sitCH4(i,1:3)
  !enddo
  
end subroutine sites_ch4


subroutine sites_h2o (sitH2O)
  implicit none
  
  real(8), parameter :: A2bohr = 1.88972598858d0
  real(8), parameter :: Omass = 15.999d0, Hmass = 1.0079d0
  integer :: i, j, k
  real(8) :: sitH2O(26,3)
  real(8) :: unitvect(3,3)
  real(8) :: OACoor(23,3)
  real(8) :: com(3)
  real(8) :: rOH0(3), rOH(3), scaleFact(3)
  
  ! Compute center of mass
  com(1:3) = (Omass*sitH2O(1,1:3) + Hmass*sitH2O(2,1:3) + Hmass*sitH2O(3,1:3))/(Omass + 2*Hmass)
  
  ! z-axis is in direction from com to O1
  unitvect(3,1:3) = sitH2O(1,1:3) - com(1:3)
  unitvect(3,1:3) = unitvect(3,1:3)/sqrt(unitvect(3,1)**2 + unitvect(3,2)**2 + unitvect(3,3)**2)
  ! x-axis is in direction from com to H2, minus the projection onto z-axis
  unitvect(1,1:3) = sitH2O(3,1:3) - com(1:3)
  unitvect(1,1:3) = unitvect(1,1:3) - unitvect(3,1:3)*( &
       unitvect(1,1)*unitvect(3,1) + unitvect(1,2)*unitvect(3,2) + unitvect(1,3)*unitvect(3,3))
  unitvect(1,1:3) = unitvect(1,1:3)/sqrt(unitvect(1,1)**2 + unitvect(1,2)**2 + unitvect(1,3)**2)
  ! y-axis is z cross x
  unitvect(2,1) = unitvect(3,2)*unitvect(1,3) - unitvect(3,3)*unitvect(1,2)
  unitvect(2,2) = unitvect(3,3)*unitvect(1,1) - unitvect(3,1)*unitvect(1,3)
  unitvect(2,3) = unitvect(3,1)*unitvect(1,2) - unitvect(3,2)*unitvect(1,1)
  
  ! Off-atomic site locations in the original coordinate system
  OACoor( 1,1:3) = [ 0.0000000000d0,-0.2067212833d0,-0.2475031107d0]
  OACoor( 2,1:3) = [ 0.0000000000d0, 0.2067212833d0,-0.2475031107d0]
  OACoor( 3,1:3) = [ 0.8995949545d0, 0.0530000110d0, 0.6431605974d0]
  OACoor( 4,1:3) = [-0.8995949545d0, 0.0530000110d0, 0.6431605974d0]
  OACoor( 5,1:3) = [ 0.8995949545d0,-0.0530000110d0, 0.6431605974d0]
  OACoor( 6,1:3) = [-0.8995949545d0,-0.0530000110d0, 0.6431605974d0]
  OACoor( 7,1:3) = [ 1.2894195922d0, 0.1399999719d0,-0.9145430242d0]
  OACoor( 8,1:3) = [-1.2894195922d0, 0.1399999719d0,-0.9145430242d0]
  OACoor( 9,1:3) = [ 1.2894195922d0,-0.1399999719d0,-0.9145430242d0]
  OACoor(10,1:3) = [-1.2894195922d0,-0.1399999719d0,-0.9145430242d0]
  OACoor(11,1:3) = [ 0.5597480766d0, 0.2399999249d0,-0.4824059566d0]
  OACoor(12,1:3) = [-0.5597480766d0, 0.2399999249d0,-0.4824059566d0]
  OACoor(13,1:3) = [ 0.5597480766d0,-0.2399999249d0,-0.4824059566d0]
  OACoor(14,1:3) = [-0.5597480766d0,-0.2399999249d0,-0.4824059566d0]
  OACoor(15,1:3) = [ 0.9095905065d0, 0.2099999579d0,-0.6833998620d0]
  OACoor(16,1:3) = [-0.9095905065d0, 0.2099999579d0,-0.6833998620d0]
  OACoor(17,1:3) = [ 0.9095905065d0,-0.2099999579d0,-0.6833998620d0]
  OACoor(18,1:3) = [-0.9095905065d0,-0.2099999579d0,-0.6833998620d0]
  OACoor(19,1:3) = [ 1.4793339462d0, 0.2600000288d0,-0.6231017284d0]
  OACoor(20,1:3) = [-1.4793339462d0, 0.2600000288d0,-0.6231017284d0]
  OACoor(21,1:3) = [ 1.4793339462d0,-0.2600000288d0,-0.6231017284d0]
  OACoor(22,1:3) = [-1.4793339462d0,-0.2600000288d0,-0.6231017284d0]
  OACoor(23,1:3) = [ 0.0000000000d0, 0.0000000000d0,-0.2436853139d0]
  
  ! Find mean O-H distance in new coordinate system
  rOH(:) = 0.0d0
  do i = 2,3
     rOH0(1:3) = sitH2O(i,1:3) - sitH2O(1,1:3)
     do j = 1,3
        rOH(j) = rOH(j) + abs(rOH0(1)*unitvect(j,1) + rOH0(2)*unitvect(j,2) + rOH0(3)*unitvect(j,3))
     enddo
  enddo
  rOH(:) = rOH(:)/2
  
  ! Scale factors in x and z directions are ratios of O-H distances to those of the original geometry
  scaleFact(1) = rOH(1)/1.452998d0 !1.4536519622d0
  scaleFact(2) = 1.0d0
  scaleFact(3) = rOH(3)/1.127262d0 !1.1216873242d0
  
  ! Compute off-atomic site locations in new coordinate system, with origin at com
  do i = 1,23
     sitH2O(i+3,1:3) = com(1:3)
     do j = 1,3
        sitH2O(i+3,1:3) = sitH2O(i+3,1:3) + scaleFact(j)*unitvect(j,1:3)*OACoor(i,j)
     enddo
  enddo
  
  !do i = 1,26
  !   write(*,'(3F20.10)') sitH2O(i,1:3)
  !enddo
  
end subroutine sites_h2o

